/*
 * Game Template
 * Copyright (C) 2007 Michael Birken
 * 
 * This file is part of Game Template.
 *
 * Game Template is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published 
 * by the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Game Template is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class GameTemplate extends JFrame {

  private static final int WIDTH = 640;
  private static final int HEIGHT = 480;
  private static final int FRAMES_PER_SECOND = 60;
  private static final long FRAME_PERIOD = 1000000000L / FRAMES_PER_SECOND;
  
  private final boolean[] KEYS = new boolean[65535]; // pressed keys
  
  private final int BALLS = 5;
  private final int BALL_SIZE = WIDTH / 10;
  private final int PADDLE_WIDTH = BALL_SIZE * 2;
  private final int PADDLE_HEIGHT = HEIGHT / 20;
  private final int BALL_VELOCITY = 2;
  private final int PADDLE_VELOCITY = 10;
  private JPanel panel;
  private Image image;
  private Graphics imageGraphics;
  private Random random = new Random();
  private Ball[] balls = new Ball[BALLS];
  private int paddleX;  
  
  public GameTemplate() {   
    initFrame();
    initImage();
    initModel();
    executeGameLoop();
  }
  
  private void executeGameLoop() {
    long nextFrameStart = System.nanoTime();
    while(true) {
      do {
        updateModel();
        nextFrameStart += FRAME_PERIOD;
      } while(nextFrameStart < System.nanoTime());     
      renderFrame();
      long remaining = nextFrameStart - System.nanoTime();
      if (remaining > 0) {
        try {
          Thread.sleep(remaining / 1000000);
        } catch(Throwable t) {          
        }
      }
    }
  }
  
  private void updateModel() {
    // Move the balls  
    for(int i = 0; i < BALLS; i++) {
      Ball ball = balls[i];
      if (ball.x >= WIDTH - BALL_SIZE) {
        ball.vx = -BALL_VELOCITY;
      } else if (ball.x <= 0) {
        ball.vx = BALL_VELOCITY;
      }
      if (ball.y >= HEIGHT - BALL_SIZE - PADDLE_HEIGHT) {
        ball.vy = -BALL_VELOCITY;
      } else if (ball.y <= 0) {
        ball.vy = BALL_VELOCITY;
      }     
      ball.x += ball.vx;
      ball.y += ball.vy;
    }

    // Move the paddle
    if (KEYS[KeyEvent.VK_LEFT] && paddleX > 0) {
      paddleX -= PADDLE_VELOCITY;
    }
    if (KEYS[KeyEvent.VK_RIGHT] && paddleX < WIDTH - PADDLE_WIDTH) {
      paddleX += PADDLE_VELOCITY;
    }
  }
  
  private void renderFrame() {
    imageGraphics.setColor(Color.BLACK);
    imageGraphics.fillRect(0, 0, WIDTH, HEIGHT);

    // Render balls
    imageGraphics.setColor(Color.RED);
    for(int i = 0; i < BALLS; i++) {
      Ball ball = balls[i];
      imageGraphics.fillOval(ball.x, ball.y, BALL_SIZE, BALL_SIZE);
    }

    // Render paddle
    imageGraphics.setColor(Color.YELLOW);
    imageGraphics.fillRect(
        paddleX, HEIGHT - PADDLE_HEIGHT, PADDLE_WIDTH, PADDLE_HEIGHT);

    // Draw to screen
    Graphics panelGraphics = panel.getGraphics();
    if (panelGraphics != null) {
      panelGraphics.drawImage(image, 0, 0, null);
      panelGraphics.dispose();
    }
  }
  
  private void initFrame() {
    setTitle("Game Template");
    setResizable(false);
    panel = (JPanel)getContentPane();
    panel.setPreferredSize(new Dimension(WIDTH, HEIGHT));    
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    pack();
    setLocationRelativeTo(null);
    setVisible(true); 
  }
  
  private void initImage() {
    image = createImage(WIDTH, HEIGHT);
    imageGraphics = image.getGraphics();
  }
  
  private void initModel() {
    for(int i = 0; i < BALLS; i++) {
      balls[i] = new Ball( 
        random.nextInt(WIDTH - BALL_SIZE),
        random.nextInt(HEIGHT - BALL_SIZE - PADDLE_HEIGHT),
        random.nextBoolean() ? -BALL_VELOCITY : BALL_VELOCITY,
        random.nextBoolean() ? -BALL_VELOCITY : BALL_VELOCITY
      );
    }
    paddleX = (WIDTH - PADDLE_WIDTH) / 2;
  }
  
  public void processKeyEvent(KeyEvent e) {
    KEYS[e.getKeyCode()] = e.getID() == 401;
  }
  
  public static void main(String[] args) {
    new GameTemplate();
  }
}

class Ball {
  public int x;  
  public int y;
  public int vx;
  public int vy;
  
  public Ball(int x, int y, int vx, int vy) {
    this.x = x;
    this.y = y;
    this.vx = vx;
    this.vy = vy;
  }
}

